Prepare Raspbian SD
===================

## Propose:

Create your `Raspbian SD Card` from standard [raspberrypi.org](https://downloads.raspberrypi.org/) images
and do some custom configuration.

-----

**Warning**

* `Raspbian` has a new name: `Raspberry Pi OS`
* This project attempts to manage the transition smoothly.
* 64 bits version is still under beta testing on [raspberrypi.org](https://raspberrypi.org/downloads) web site. Web site didn't provide any information that could be parse to get last 64 bits version - There is only one 64 image at this time.

```bash
# How to install 64 bit beta version ?
beta_url=https://downloads.raspberrypi.org/raspios_arm64/images/raspios_arm64-2020-05-28/2020-05-27-raspios-buster-arm64.zip
beta_zip_file="$( basename "${beta_url}" )"
beta_img_file="${beta_zip_file%.zip}.img"
prepare-raspbian-sd --url "${beta_url}" --expected-img "${beta_img_file}" --unzip
```

-----

These tools are design for people who have many Raspberry Pis and want to rebuild then with an automated
process.

More details:

* Download image version (optional)
* Initialize SD
* Customize SD (optional)
    * Set memory split (currently fixed to 16)
    * Prepare [bcm workspace](https://git.cclaude.rocks/system/bcm-runtime)
    * Enable ssh
    * Add public keys to `PI_USER_HOME/.ssh/authorized_keys`.

## Requirement:

* GNU Linux (fully tested under Mint)
* Some classic commands are required and will be installed on your system:
    * `curl`
    * `jq`


How to clone this repository
============================

Since this repository use **git sub-modules** you need to initialize and update embedded modules:

```bash
git clone --recurse-submodules https://git.cclaude.rocks/raspbian/prepare-raspbian-sd.git
```

Then follow installation section.


How to update this repository
=============================

Since this repository use **git sub-modules** you need to initialize (if new sub-modules have been added)
and update embedded modules:

```bash
git fetch --verbose --prune # refresh git references
git pull --ff-only          # update content of current branch (if needed)
                            # don't merge with your modifications (if any)
git submodule init          # Initialize git sub-module (if needed)
git submodule update        # Update git sub-module (if needed)
git status                  # Show current status
```

Then follow installation section.

You can also just delete completely previous version and clone again

```bash
rm -frv prepare-raspbian-sd &&
git clone --recurse-submodules https://git.cclaude.rocks/raspbian/prepare-raspbian-sd.git
```


Installation
------------

When you have a copy of a version, or a clone run :

```bash
# cd PREPARE_RASPBIAN_SD_DIRECTORY
sudo bash prepare-raspbian-sd-install.sh
```

To remove the applications, use:

```bash
sudo bash /usr/local/share/prepare-raspbian-sd/prepare-raspbian-sd-remove.sh
```

### Install options

You can get help using :

```bash
bash ./prepare-raspbian-sd-install.sh --help
```

| Option             | Description                   |
|--------------------|-------------------------------|
| -S, --skip-update  | To skip update before install required packets |


Configuration
-------------

Optionally you can customize **Prepare Raspbian SD**.

You have two options to customize default download directory, witch is location where images will be store.
First you can use `prepare-raspbian-sd-config`:

```bash
_DATA_="$( xdg-user-dir DOCUMENTS )/raspbian-images"
_DOWNLOAD_="${_DATA_}/download"
_BACKUP_="${_DATA_}/backups"

# Create directories if needed
mkdir -vp "${_DOWNLOAD_}" "${_BACKUP_}"

# Configuration (recommended)
prepare-raspbian-sd-config RASPBIAN_DEFAULT_BACKUP_DIRECTORY "${_BACKUP_}"
prepare-raspbian-sd-config RASPBIAN_DEFAULT_DOWNLOAD_IMAGE_DIRECTORY "${_DOWNLOAD_}"
prepare-raspbian-sd-config SSH_AUTHORIZED_KEYS "$( cat ~/.ssh/id_rsa.pub )"

# Configuration (optional - if used ensure value is what you want)
prepare-raspbian-sd-config DEFAULT_SD_DEV_NAME 'mmcblk0'
prepare-raspbian-sd-config GPU_MEM '16'
prepare-raspbian-sd-config RASPBIAN_DEFAULT_IMG "${_DOWNLOAD_}/2019-07-10-raspbian-buster-lite.img"
```

Or you can export required variables (using `~/.bashrc` or custom script):

```bash
export RASPBIAN_SSH_CONFIG='/path/to/your-ssh-need.config'
```

* If `RASPBIAN_DEFAULT_BACKUP_DIRECTORY`  not define, commands your `Documents` directory.
* If `RASPBIAN_DEFAULT_DOWNLOAD_IMAGE_DIRECTORY` not define, commands use your `Download` directory.


### Summary

| Variable name                               | Usage description                         | Typical values |
|---------------------------------------------|-------------------------------------------|----------------|
| `DEFAULT_SD_DEV_NAME`                       | Optional, use to write on SD card.        | `mmcblk0`      |
| `GPU_MEM`                                   | Optional, use by `--configure`. Configure memory split, if not set or if set to 0 memory split configuration is skipped | `16` |
| `RASPBIAN_DEFAULT_BACKUP_DIRECTORY`         | Required to be able to use `--full-clone`. Variable must contain full path of an existing directory where your SD backups will be stored. | `/home/username/Documents/` |
| `RASPBIAN_DEFAULT_DOWNLOAD_IMAGE_DIRECTORY` | Nice to have to use `--download` option of **prepare-raspbian-sd-download-image** and to use `--put-last-image` option of **prepare-raspbian-sd**. | `/home/username/Documents/raspbian-images` |
| `RASPBIAN_DEFAULT_IMG`                      | Required to be able to use `--put-default-image`. Variable must contain full path to your `raspbian-stretch-lite.img` local file. | `/path/to/2018-10-09-raspbian-stretch-lite.img` |
| `RASPBIAN_PI_GROUP_ID`                      | Optional, default value 1000              | `1000` |
| `RASPBIAN_PI_USER_ID`                       | Optional, default value 1000              | `1000` |
| `SSH_AUTHORIZED_KEYS`                       | Required to be able to use `--configure`. Variable must contain content of `/home/pi/.ssh/authorized_keys` you want to add. | `"$( cat ~/.ssh/id_rsa.pub )"` |

Deprecated configuration entry point:

| Variable name                               | Usage description                                 |
|---------------------------------------------|---------------------------------------------------|
| `RASPBIAN_SSH_CONFIG`                       | Is deprecated. Use `SSH_AUTHORIZED_KEYS` instead. |


Usage
-----

To prevent mistakes the full process needed two steps:

### Initialize SD

SD must be plug in your computer but **not mounted**.

Download if needed last Raspbian image, then initialize SD using this image.

```bash
prepare-raspbian-sd --put-last-image
```

or just:

```bash
prepare-raspbian-sd -L
```

**Warning**

New versions of `Raspbian` is are called `Raspberry Pi OS`, to get **new versions**,
you must use following command:

```bash
prepare-raspbian-sd -L --arch armhf # 32 bits version (if your PI have 512 Mo, 1 Go, 2 Go or 4 GO of RAM)
prepare-raspbian-sd -L --arch arm64 # 64 bits version (if your PI have more than 4 GO of RAM)
```

If you want to use desktop version of `Raspberry Pi OS` (which is the default solution)
on [raspberrypi.org](https://raspberrypi.org/) web site, you must use:

```bash
prepare-raspbian-sd -L --full --arch armhf
```

**The syntax will change in the future,
since `Raspberry Pi OS` will be available in 32-bit and 64-bit versions.**


### Customize SD

This optional step, customize SD:

* Enable **ssh** by creating a empty `ssh` file in root folder of boot partition.
* Set memory split to minimum (16 MB) by adding related configuration to `config.txt` in root folder of boot partition.
* Customize **pi user** home directory:
    * Create folder `.bcm-workspace` for [bcm-runtime](https://git.cclaude.rocks/system/bcm-runtime) project,
    * Create folder `.ssh` with secure configuration, and generate `.ssh/authorized_keys` if `SSH_AUTHORIZED_KEYS` is defined.

SD must be plug in your computer and **mounted**.

Then to customize SD. At the end of process card is sync and unmounted.

```bash
prepare-raspbian-sd --configure
```

### Initialize SD using custom URL

For instance to prepare an SD card based on [LibreElec](https://libreelec.tv/downloads/):

```bash
prepare-raspbian-sd --url http://releases.libreelec.tv/LibreELEC-RPi2.arm-8.2.5.img.gz --expected-img LibreELEC-RPi2.arm-8.2.5.img
```

Note that **LibreELEC** is supported. You can use `prepare-libreelec-sd` to initialiaze your
SD card with last **LibreELEC** image.

```bash
prepare-libreelec-sd
```

`prepare-libreelec-sd` is a wrapper for `prepare-raspbian-sd`.


### Download and unzip image using custom URL (SD card not required)

Both `*.gz` and `*.zip` file are supported.

You can use custom URL

```bash
prepare-raspbian-sd-download-image --download --url http://releases.libreelec.tv/LibreELEC-RPi2.arm-8.2.5.img.gz --expected-img LibreELEC-RPi2.arm-8.2.5.img --unzip
```

```bash
prepare-raspbian-sd-download-image --download --url https://downloads.raspberrypi.org/raspbian_lite/images/raspbian_lite-2019-07-12/2019-07-10-raspbian-buster-lite.zip --expected-img 2019-07-10-raspbian-buster-lite.img --unzip
```


### Create image of your SD

To create an image of your SD, use following syntax:

```bash
prepare-raspbian-sd -T ~/my-sd-backup.img
```

TO-DO list
----------

* [ ] Configuration: Use JSON to store config
* [X] Configuration: Allow to change memory split value.
* [ ] Configuration: Allow to not enable ssh
* [ ] Configuration: Allow to not create `.ssh` directory
* [ ] Configuration: Allow to not create `.bcm-workspace` directory
* [ ] Allow to use directly zip file when creating image (bench needed)(Is this useful ?)


Related projects
----------------

* [bcm-runtime](https://git.cclaude.rocks/system/bcm-runtime) project is design to manage your Linux (Raspbian, ...) hosts through ssh.

