#!/bin/bash
#
# prepare-raspbian-sd:bin/prepare-libreelec-sd.sh
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -gr PROJECT_HOME

# shellcheck disable=SC1090
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit 1
# shellcheck disable=SC1090
source "${PROJECT_HOME}/bin/prepare-raspbian-api.sh" || exit 1

function usage {
  cat <<EOF
Usage: $0

   $( basename "$0" ) --help
   $( basename "$0" ) [OPTIONS]...

Options:
  --download-to DIR     Upload a new image if necessary in the given directory. If the option value is
                        omitted the value defined by RASPBIAN_DEFAULT_DOWNLOAD_IMAGE_DIRECTORY will be used.
                        RASPBIAN_DEFAULT_DOWNLOAD_IMAGE_DIRECTORY:
                        '$( print_variable RASPBIAN_DEFAULT_DOWNLOAD_IMAGE_DIRECTORY )'
  --dev-name DEVNAME    Forces the name of the device to be used for the SD card.
                        Default value: '$( print_variable DEFAULT_SD_DEV_NAME )'
  --expected-img IMG    Expected image file name (no path). Mandatory with action --url does not download
                        the remote file if a local file is present.

EOF
}

function getLastLibreElecUrl {
  # Previous URL:
  #
  #     download_url='https://libreelec.tv/downloads/'
  #
  local -r download_url='https://libreelec.tv/downloads_new/raspberry-pi-3-3/' # URLs for Rasberry 3 (Customize this?)

  curl "${download_url}" | grep '.img.gz"' | sed -e 's/<a href="/\n/g' | cut -d '"' -f1 | grep '.img.gz$' | sort -r | head -n1

  # Expect something like:
  #
  #     http://releases.libreelec.tv/LibreELEC-RPi2.arm-9.2.1.img.gz
  #
}

function prepare_libreelec_sd {
  local -a params=()
  local    url=

  url="$( getLastLibreElecUrl )" || exit 1

  params+=(--unzip)
  params+=(--url)
  params+=("${url}")
  echoinfo "URL: '${url}'"

  if [ -z "${param_expected_img}" ] ; then
    param_expected_img="$( basename "${url%.gz}" )" || exit 1
  fi

  params+=(--expected-img)
  params+=("${param_expected_img}")
  echoinfo "FILE: '${param_expected_img}'"

  if [ -n "${param_download_to}" ] ; then
    params+=(--download-to)
    params+=("${param_download_to}")
  fi

  if [ -n "${param_dev_name}" ] ; then
    params+=(--dev-name)
    params+=("${param_dev_name}")
  fi

  cat <<EOF >&2
  "${PROJECT_HOME}/bin/prepare-raspbian-sd.sh" "${params[@]}"
EOF
  "${PROJECT_HOME}/bin/prepare-raspbian-sd.sh" "${params[@]}" || exit 1
}

function main {
  local param=
  local param_dev_name=
  local param_download_to=
  local param_expected_img=

  while [ $# -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
    '-h'|'--help')
      usage
      exit 0
      ;;

    '--download-to')
      param_download_to="$( P "$@" )" && shift
      ;;

    '--dev-name')
       param_dev_name="$( P "$@" )" && shift
       ;;

    '--expected-img')
      param_expected_img="$( P "$@" )" && shift
      ;;

    *)
      # unknown option
      echoerror "Undefine parameter '${param}'."
      usage >&2
      exit 1
      ;;
    esac
  done

  prepare_libreelec_sd
}

main "$@"

