#!/bin/bash -e
#
# prepare-raspbian-sd:prepare-raspbian-sd-install.sh
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )"
declare -r PROJECT_HOME

# shellcheck disable=SC1090
source "${PROJECT_HOME}/bin/install-config.sh" || exit 1
# shellcheck disable=SC1090
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit 1
# shellcheck disable=SC1090
source "${PROJECT_HOME}/apis/install-api/install-api.sh" || exit 1

function usage {
  cat <<EOF
Usage:

  $( basename "${BASH_SOURCE[0]}" ) --help
  $( basename "${BASH_SOURCE[0]}" ) [--skip-update]

Options:
  -h,       --help  Display help message and exit.
  -S,       --skip-update
                    Skip package manager update

EOF
}

function main {
  local param_skip_update=

  while [ $# -gt 0 ] ; do
    local param="$1"
    shift

    case "${param}" in
      '-h'|'--help') usage ; exit 0 ;;
      '-S'|'--skip-update') param_skip_update='skip_update' ;;

      *)
        echo "Unknown option ${param}" >&2
        usage >&2
        exit 1
        ;;
    esac
  done

  exit_if_not_root

  install_if_needed "${param_skip_update}" '' "${INSTALL_PACKAGES[@]}"

  remove_previous_version_before_install

  install_main
}

main "$@"
